from functools import wraps
from flask import request

from jwt_library.jwt_operations.okta.jwt import verify_token

def jwt_required(iss, aud, client_ids):
   def decorator(func):
       @wraps(func)
       def wrapper(*args, **kwargs):
            try:
                token = request.headers.get("Authorization")
                payload = verify_token(token, iss=iss, aud=aud, client_ids=client_ids)
            except Exception as e:
                return str(e), 401
            if payload:
                return func(*args, **kwargs)
       return wrapper
   return decorator


from functools import wraps
import json
from six.moves.urllib.request import urlopen
from flask import request, _request_ctx_stack


from jwt_library.jwt_operations.auth0 import get_payload
from jwt_library.jwt_operations.auth0.jwt_operations import fetch_token_from_header, AuthError, get_unverified_header, validate_algorithm
from jwt_library.jwt_operations.utils.token_validation import validate_signature, validate_claims
from jose import jwt


def jwt_required_auth0(iss, aud):
    def requires_auth(f):
        """Determines if the access token is valid """
        @wraps(f)
        def decorated(*args, **kwargs):
            try:
                token = fetch_token_from_header(request.headers.get("Authorization"))
                jsonurl = urlopen("https://"+ iss +"/.well-known/jwks.json")
                jwks = json.loads(jsonurl.read())
                unverified_header = get_unverified_header(token)

                validate_algorithm(unverified_header=unverified_header)

                rsa_key = {}
                for key in jwks["keys"]:
                    if key["kid"] == unverified_header["kid"]:
                        rsa_key = key
                if rsa_key and \
                validate_claims(get_payload(token=token, rsa_key=rsa_key, issuer=iss, audience=aud), issuer=iss, audience=aud) and \
                validate_signature(token, rsa_key):
                        return f(*args, **kwargs)
                raise AuthError({"status": "invalid_header",
                            "message": "Unable to find appropriate key"}, 401)
            except AuthError as ae:
                return dict(ae.error), 401
        return decorated
    return requires_auth


from jose import jwt

from jwt_library import ALGORITHMS
from jwt_library.jwt_operations.utils.errors import AuthError

def fetch_token_from_header(token):
    if not token:
        raise AuthError({ "status": "authorization_header_missing",
                          "message": "Authorization header is expected."
        }, 401)

    parts = token.split()

    if parts[0].lower() != 'bearer':
        raise AuthError({"status": "invalid_header",
                        "message":
                            "Authorization header must start with"
                            " Bearer"}, 401)
    elif len(parts) == 1:
        raise AuthError({"status": "invalid_header",
                        "message": "Token not found"}, 401)
    elif len(parts) > 2:
        raise AuthError({"status": "invalid_header",
                        "message":
                            "Authorization header must be"
                            " Bearer token"}, 401)

    token = parts[1]
    return token

def validate_algorithm(unverified_header):
    if unverified_header['alg'] == "HS256":
            raise AuthError({"code": "invalid_header",
                "message":
                    "Invalid header. "
                    "Use an RS256 signed JWT Access Token"}, 401)

def get_unverified_header(token):
    try:
        unverified_header = jwt.get_unverified_header(token)
    except jwt.JWTError:
        raise AuthError({"code": "invalid_header",
                        "message":
                            "Invalid header. "
                            "Use an RS256 signed JWT Access Token"}, 401)
    return unverified_header

def get_payload(token, rsa_key, issuer, audience):
    try:
        payload = jwt.decode(
            token,
            rsa_key,
            algorithms=ALGORITHMS,
            audience=audience,
            issuer="https://"+issuer+"/"
        )
    except jwt.ExpiredSignatureError:
        raise AuthError({"status": "token_expired",
                        "message": "token is expired"}, 401)
    except jwt.JWTClaimsError:
        raise AuthError({"status": "invalid_claims",
                        "message":
                            "incorrect claims,"
                            " please check the audience and issuer"}, 401)
    except Exception:
        raise AuthError({"status": "invalid_header",
                        "message":
                            "Unable to parse authentication"
                            " token."}, 401)
    return payload



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="contractor-jwt-lib",
    version="0.0.12",
    author="Daniel Gellman",
    description="Internal library for managing JWTs",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/pypa/sampleproject",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
    install_requires=['certifi>=2020.4.5.1',
                        'chardet>=3.0.4',
                        'ecdsa>=0.15',
                        'idna>=2.9',
                        'okta-jwt>=1.3.5',
                        'pyasn1>=0.4.8',
                        'python-jose>=3.1.0',
                        'requests>=2.23.0',
                        'rsa>=4.0',
                        'six>=1.14.0',
                        'urllib3>=1.25.9',
],
    include_package_data=True,
    zip_safe=False
)



from calendar import timegm
from datetime import datetime, timedelta
from jose import jwk
from jose.exceptions import JWTClaimsError, ExpiredSignatureError
from jose.utils import base64url_decode
from six import string_types

def verify_issuer(payload, issuer):
    if payload['iss'] != "https://" + issuer + "/":
        raise JWTClaimsError('Invalid Issuer')

def verify_cid(payload, cid_list):
    if not payload['cid'] in cid_list:
        raise JWTClaimsError('Invalid Client')

def verify_expiration(payload, leeway=0):
    if 'exp' not in payload:
        return

    try:
        exp = int(payload['exp'])
    except ValueError:
        raise JWTClaimsError('Expiration Time payload (exp) must be an integer.')

    now = timegm(datetime.utcnow().utctimetuple())

    if exp < (now - leeway):
        raise ExpiredSignatureError('Token is expired.')

def verify_audience(payload, audience=None):
    if 'aud' not in payload:
        return


    audience_c = payload['aud']

    if any(not isinstance(c, string_types) for c in audience_c):
        raise JWTClaimsError('Invalid claim format in token')
    if audience not in audience_c:
        raise JWTClaimsError('Invalid Audience')

def verify_iat(payload, leeway=300):
    time_now_with_leeway = datetime.utcnow() + timedelta(seconds=leeway)
    acceptable_iat = timegm((time_now_with_leeway).timetuple())

    if 'iat' in payload and payload['iat'] > acceptable_iat:
        raise JWTClaimsError('Invalid Issued At(iat) Time')

def validate_signature(token: str, rsa_key: dict) -> bool:
    """[summary]

    Args:
        token ([type]): [description]
    """
    key = jwk.construct(rsa_key)
    message, encoded_sig = token.rsplit('.', 1)
    decoded_sig = base64url_decode(encoded_sig.encode('utf-8'))
    valid = key.verify(message.encode(), decoded_sig)
    return valid

def validate_claims(payload, issuer, audience, cid_list=None):
    """ Validates Issuer, Client IDs, Audience
    Issued At time and Expiration in the Payload
    """
    if cid_list:
        verify_cid(payload, cid_list)

    verify_issuer(payload, issuer)
    verify_audience(payload, audience)
    verify_expiration(payload)
    verify_iat(payload)
    return True